package sun.awt.windows;

import sun.awt.AWTAccessor;
import java.awt.ScrollPaneAdjustable;
import sun.awt.PeerEvent;
import sun.awt.SunToolkit;
import java.awt.Dimension;
import java.awt.ScrollPane;
import java.awt.Point;
import java.awt.Adjustable;
import java.awt.Insets;
import java.awt.Component;
import sun.util.logging.PlatformLogger;
import java.awt.peer.ScrollPanePeer;

final class WScrollPanePeer extends WPanelPeer implements ScrollPanePeer
{
    private static final PlatformLogger log;
    int scrollbarWidth;
    int scrollbarHeight;
    int prevx;
    int prevy;
    
    static native void initIDs();
    
    @Override
    native void create(WComponentPeer p0);
    
    native int getOffset(int p0);
    
    WScrollPanePeer(Component component) {
        super(component);
        this.scrollbarWidth = this._getVScrollbarWidth();
        this.scrollbarHeight = this._getHScrollbarHeight();
    }
    
    @Override
    void initialize() {
        super.initialize();
        this.setInsets();
        Insets insets = this.getInsets();
        this.setScrollPosition(-insets.left, -insets.top);
    }
    
    @Override
    public void setUnitIncrement(Adjustable adjustable, int n) {
    }
    
    @Override
    public Insets insets() {
        return this.getInsets();
    }
    
    private native void setInsets();
    
    @Override
    public synchronized native void setScrollPosition(int p0, int p1);
    
    @Override
    public int getHScrollbarHeight() {
        return this.scrollbarHeight;
    }
    
    private native int _getHScrollbarHeight();
    
    @Override
    public int getVScrollbarWidth() {
        return this.scrollbarWidth;
    }
    
    private native int _getVScrollbarWidth();
    
    public Point getScrollOffset() {
        return new Point(this.getOffset(0), this.getOffset(1));
    }
    
    @Override
    public void childResized(int n, int n2) {
        Dimension size = ((ScrollPane)this.target).getSize();
        this.setSpans(size.width, size.height, n, n2);
        this.setInsets();
    }
    
    synchronized native void setSpans(int p0, int p1, int p2, int p3);
    
    @Override
    public void setValue(Adjustable adjustable, int n) {
        Component scrollChild = this.getScrollChild();
        if (scrollChild == null) {
            return;
        }
        Point location = scrollChild.getLocation();
        switch (adjustable.getOrientation()) {
            case 1: {
                this.setScrollPosition(-location.x, n);
                break;
            }
            case 0: {
                this.setScrollPosition(n, -location.y);
                break;
            }
        }
        return;
    }
    
    private Component getScrollChild() {
        ScrollPane scrollPane = (ScrollPane)this.target;
        Component component = null;
        try {
            component = scrollPane.getComponent(0);
        }
        catch (ArrayIndexOutOfBoundsException ex) {}
        return component;
    }
    
    private void postScrollEvent(int n, int n2, int n3, boolean b) {
        SunToolkit.executeOnEventHandlerThread(new WScrollPanePeer.ScrollEvent(this.target, new WScrollPanePeer.Adjustor(n, n2, n3, b)));
    }
    
    static Component access$100(WScrollPanePeer wScrollPanePeer) {
        return wScrollPanePeer.getScrollChild();
    }
    
    static {
        log = PlatformLogger.getLogger("sun.awt.windows.WScrollPanePeer");
        initIDs();
    }
    
    class ScrollEvent extends PeerEvent
    {
        ScrollEvent(Object source, Runnable runnable) {
            super(source, runnable, 0L);
        }
        
        @Override
        public PeerEvent coalesceEvents(PeerEvent obj) {
            if (WScrollPanePeer.log.isLoggable(PlatformLogger.Level.FINEST)) {
                WScrollPanePeer.log.finest("ScrollEvent coalesced: " + obj);
            }
            if ((obj instanceof WScrollPanePeer.ScrollEvent)) {
                return obj;
            }
            return null;
        }
    }
    
    class Adjustor implements Runnable
    {
        int orient;
        int type;
        int pos;
        boolean isAdjusting;
        
        Adjustor(int orient, int type, int pos, boolean isAdjusting) {
            this.orient = orient;
            this.type = type;
            this.pos = pos;
            this.isAdjusting = isAdjusting;
        }
        
        @Override
        public void run() {
            if (WScrollPanePeer.access$100(WScrollPanePeer.this) == null) {
                return;
            }
            ScrollPane scrollPane = (ScrollPane)WScrollPanePeer.this.target;
            ScrollPaneAdjustable scrollPaneAdjustable = null;
            if (this.orient == 1) {
                scrollPaneAdjustable = (ScrollPaneAdjustable)scrollPane.getVAdjustable();
            }
            else if (this.orient == 0) {
                scrollPaneAdjustable = (ScrollPaneAdjustable)scrollPane.getHAdjustable();
            }
            else if (WScrollPanePeer.log.isLoggable(PlatformLogger.Level.FINE)) {
                WScrollPanePeer.log.fine("Assertion failed: unknown orient");
            }
            if (scrollPaneAdjustable == null) {
                return;
            }
            int value = scrollPaneAdjustable.getValue();
            int pos;
            switch (this.type) {
                case 2: {
                    pos = value - scrollPaneAdjustable.getUnitIncrement();
                    break;
                }
                case 1: {
                    pos = value + scrollPaneAdjustable.getUnitIncrement();
                    break;
                }
                case 3: {
                    pos = value - scrollPaneAdjustable.getBlockIncrement();
                    break;
                }
                case 4: {
                    pos = value + scrollPaneAdjustable.getBlockIncrement();
                    break;
                }
                case 5: {
                    pos = this.pos;
                    break;
                }
                default: {
                    if (WScrollPanePeer.log.isLoggable(PlatformLogger.Level.FINE)) {
                        WScrollPanePeer.log.fine("Assertion failed: unknown type");
                    }
                    return;
                }
            }
            int min = Math.min(scrollPaneAdjustable.getMaximum(), Math.max(scrollPaneAdjustable.getMinimum(), pos));
            scrollPaneAdjustable.setValueIsAdjusting(this.isAdjusting);
            AWTAccessor.getScrollPaneAdjustableAccessor().setTypedValue(scrollPaneAdjustable, min, this.type);
            Component component;
            for (component = WScrollPanePeer.access$100(WScrollPanePeer.this); component != null && !(component.getPeer() instanceof WComponentPeer); component = component.getParent()) {}
            if (WScrollPanePeer.log.isLoggable(PlatformLogger.Level.FINE) && component == null) {
                WScrollPanePeer.log.fine("Assertion (hwAncestor != null) failed, couldn't find heavyweight ancestor of scroll pane child");
            }
            ((WComponentPeer)component.getPeer()).paintDamagedAreaImmediately();
            return;
        }
    }
}
